##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

'''$Id: db.py,v 1.6 2005/05/13 21:55:11 nakagami Exp $'''
__version__='$Revision: 1.6 $'[11:-2]

import os, thread
from string import strip, split
from pysqlite2 import dbapi2 as pysqlite
import Globals, Shared.DC.ZRDB.THUNK
from DateTime import DateTime

data_dir=os.path.join(Globals.data_dir,'sqlite')

def manage_DataSources():

    if not os.path.exists(data_dir):
        try:
            os.mkdir(data_dir)
        except:
            raise 'SQLite Error', (
                """
                The Zope SQLite Database Adapter requires the
                existence of the directory, <code>%s</code>.  An error
                occurred  while trying to create this directory.
                """ % data_dir)
    if not os.path.isdir(data_dir):
        raise 'SQLite Error', (
            """
            The Zope SQLite Database Adapter requires the
            existence of the directory, <code>%s</code>.  This
            exists, but is not a directory.
            """ % data_dir)

    return map(
        lambda d: (d,''),
        filter(lambda f, i=os.path.isfile, d=data_dir, j=os.path.join:
               i(j(d,f)),
               os.listdir(data_dir))
        )

class DB(Shared.DC.ZRDB.THUNK.THUNKED_TM):

    opened=''

    def _get_db_connect(self):
        filename=os.path.join(data_dir,self.connection)
        thread_id = thread.get_ident()
        db = self.db_conn.get(thread_id)
        if db is None:
            try:
                self.lock.acquire()
                db = pysqlite.connect(filename)
                self.db_conn[thread_id] = db
            finally:
                self.lock.release()
        if not self.opened:
            self.opened=DateTime()
        return db

    def tables(self,*args,**kw):
        self._begin()
        c=self._get_db_connect().cursor()
        c.execute("SELECT name, type FROM sqlite_master WHERE type='table' or type='view'")

        result = []
        rs = c.fetchall()
        for r in rs:
            result.append({'TABLE_NAME':r[0], 'TABLE_TYPE':r[1]})
        self._finish()
        return result
	
    def sqlscript(self, table_name):
        self._begin()
        c=self._get_db_connect().cursor()
        c.execute("SELECT sql FROM sqlite_master WHERE name='%s'" % table_name)
        sql = c.fetchone()[0]
        self._finish()
        return sql

    def open(self):
        filename=os.path.join(data_dir,self.connection)
        try:
            self.lock.acquire()
            for thread_id in self.db_conn:
                self.db_conn[thread_id] = sqlite.connect(filename)
        finally:
            self.lock.release()
        self.opened=DateTime()

    def close(self):
        try:
            self.lock.acquire()
            for thread_id in self.db_conn:
                self.db_conn[thread_id] = None
        finally:
            self.lock.release()
        self.opened = None

    def __init__(self,connection):
        self.connection=connection
        self.lock = thread.allocate_lock()
        self.db_conn = {}

    def query(self,query_string, max_rows=None):
        self._register() # self._begin() calld from innter.
        c=self._get_db_connect().cursor()
        queries=filter(None, map(strip,split(query_string, '\0')))
        if not queries: raise 'Query Error', 'empty query'
        desc=None
        result=[]
        for qs in queries:
            c.execute(qs)
            d=c.description
            if d is None: continue
            if desc is None: desc=d
            elif d != desc:
                raise 'Query Error', (
                    'Multiple incompatible selects in '
                    'multiple sql-statement query'
                    )

            if max_rows:
                if not result: result=c.fetchmany(max_rows)
                elif len(result) < max_rows:
                    result=result+c.fetchmany(max_rows-len(result))
            else:
                result = c.fetchall()

        self._finish()
        if desc is None: return (),()

        items=[]
        for name, type, width, ds, p, scale, null_ok in desc:
            if type=='NUMBER':
                if scale==0: type='i'
                else: type='n'
            elif type=='DATE':
                type='d'
            else: type='s'
            items.append({
                'name': name,
                'type': type,
                'width': width,
                'null': null_ok,
                })
        return items, result

    def _begin(self):
        #pysqlite2.connection don't have begein() method.
        #self._get_db_connect().begin()
        pass

    def _finish(self):
        self._get_db_connect().commit()

    def _abort(self):
        self._get_db_connect().rollback()
